import { __awaiter, __decorate } from "tslib";
import { LifecycleMixin } from '@genesislcap/foundation-utils';
import { customElement, FoundationElement } from '@genesislcap/web-core';
import { first, Subscription } from 'rxjs';
import { FDC3 } from '../fdc3';
/**
 * fdc3-system-channel-listener
 * Util element to listen to the context on a fdc3 system channel
 * @alpha
 */
let Fdc3SystemChannelListener = class Fdc3SystemChannelListener extends LifecycleMixin(FoundationElement) {
    constructor() {
        super(...arguments);
        this.config = [];
        this.channelSub = new Subscription();
    }
    deepClone() {
        const copy = super.deepClone();
        if (this.config) {
            copy.config = this.config.map((c) => {
                const copyConfig = Object.assign({}, c);
                if (c.callback) {
                    copyConfig.callback = c.callback.bind(this);
                }
                return copyConfig;
            });
        }
        return copy;
    }
    connectedCallback() {
        super.connectedCallback();
        this.fdc3.isReady$.pipe(first()).subscribe(() => {
            if (this.config) {
                this.addChannelListeners();
            }
        });
    }
    addChannelListeners() {
        this.fdc3.isReady$.pipe(first()).subscribe(() => __awaiter(this, void 0, void 0, function* () {
            return this.config.forEach((c) => this.channelSub.add(this.fdc3.currentChannel$.subscribe((channel) => {
                this.fdc3.handleChannelCurrentContext(channel, c.channelType, c.callback);
                channel.addContextListener(c.channelType, c.callback);
            })));
        }));
    }
    disconnectedCallback() {
        var _a;
        (_a = this.channelSub) === null || _a === void 0 ? void 0 : _a.unsubscribe();
        super.disconnectedCallback();
    }
};
__decorate([
    FDC3
], Fdc3SystemChannelListener.prototype, "fdc3", void 0);
Fdc3SystemChannelListener = __decorate([
    customElement({
        name: 'fdc3-system-channel-listener',
    })
], Fdc3SystemChannelListener);
export { Fdc3SystemChannelListener };
